#![allow(dead_code)]

mod re;
mod text;

use color_eyre::{
    eyre::{
        self,
        eyre,
        WrapErr as _,
    },
    SectionExt as _, Help as _,
};

fn initialise() -> eyre::Result<()>
{
    color_eyre::install()?;
    Ok(())
}

fn print_group<S: ?Sized, G, T>(to: &mut S, g: G, group: usize) -> std::io::Result<()>
where S: std::io::Write,
      G: IntoIterator<Item = Option<T>>,
      T: std::borrow::Borrow<str>
{
    match g.into_iter().nth(group) {
	Some(None) => writeln!(to, ""),
	Some(Some(g)) => writeln!(to, "{}", g.borrow()),
	None => Ok(()),
    }
}

fn main() -> eyre::Result<()>
{
    initialise().wrap_err("Fatal: Failed to install panic handle")?;
    
    // let cli = args::parse_cli();//.wrap_err("Error parsing command-line arguments")?;
    //
    // eprintln!("{:#?}", cli);
    // return Ok(());

    let args: Vec<String> = std::env::args().collect();

    if args.len() < 4 {
	println!("Usage: {} <str> <regex> <group>", args[0]);
	println!("Pass `-' as `<str>' to read lines from stdin");
	std::process::exit(1);
    } else {
	let re = re::Regex::compile(&args[2])?;
	let text = &args[1];
	let group: usize = args[3].parse().expect("Invalid group number.");

	use std::io::Write;
	let mut stdout = std::io::stdout();
	
	if text == "-" {
	    text::stdin_lines(|text| -> eyre::Result<bool> {
		let mut stdout = stdout.lock();
		match re.exec(&text)? {
		    Some(g) if g.len() > group => print_group(&mut stdout, g, group)?, //println!("{}", &g[group]),
		    _ => (),
		}
		Ok(true)
	    })?;
	} else {

	    match re.exec(&text)? {
		Some(g) if g.len() > group => print_group(&mut stdout, g, group)?,//println!("{}", &g.nth(group).unwrap().map(|x| x.as_ref()).unwrap_or("")),
		_ => (),
	    }
	}
	stdout.flush().unwrap();
    }
    Ok(())
}
